#!/usr/bin/env python
"""

Function steps that CDBS team members manually execute to delivery reference
files for inclusion in the CDBS database.

:Author: Christopher Hanley
:Reference: TIR-CDBS-2008-02

"""

from __future__ import division

import os
import os.path
import pyfits
import error
import helper
import setlodkeywd
import uniqname

from deliverypathconfig import WORKINGDIRROOT, DELIVERYDIR

# Required for external programs
import subproc

try:
    CDBSDATA = os.environ['CDBSDATA']
except KeyError:
    CDBSDATA ="./"
    
# == Step 2 ==
def set_file_permissions(filename, log=None):
    """
    Verify  that the files have the appropriate file permissions set.
    """
    helper.updatelog(log,"\n*** set file permissions started ***\n")
    (code, out, err) = subproc.execute(["chmod", "644", filename], log=log)
    if code != 0:
        helper.updatelog(log,err)
        raise error.CDBSError(out, err, code)
    helper.updatelog(log,out)
    helper.updatelog(log,"\n*** set file permissions finished ***\n")
    return 

# == Step 3 ==
def check_headers(filename,log=None):
    """
    Verify that all relevant header keywords and history sections are
    present in the primary header of the FITS file.
    """
    helper.updatelog(log,"\n*** start check headers ***\n")
    hdulist = pyfits.open(filename)
    prihdr = hdulist[0].header
    
    # Need to check for the existence of the following
    # header keywords: pedigree, useafter, descrip, comment
    missingkeys = []
    outstr = ""
    testkeys = ['comment','descrip','useafter','pedigree']

    for testkey in testkeys:
        if prihdr.has_key(testkey):
            outstr += testkey + " : " + prihdr[testkey] + '\n'
            helper.updatelog(log,outstr)
        else:
            missingkeys.append(testkey)        
    
    if missingkeys:
        errorstr = "The file " + os.path.basename(filename) + " is missing the following "
        errorstr += "required header keywords -->   "
        errorstr += " : ".join(missingkeys)
        helper.updatelog(log,errorstr)
        raise error.CDBSError(outstr,errorstr,error.PYERRORCODE)
    helper.updatelog(log,"\n*** check headers finished ***\n")
    return

# == Step 4 ==
def check_FITS_standard(pathfilename,log=None):
    """
    Verify that all files conform to the FITS standard format.
    """
    helper.updatelog(log,"\n*** check FITS standard started ***\n")
    (fullpathfilename, path, filename) = helper.fileinfo(pathfilename)
    fixedfile = helper.fixedfilename(filename,"fixed")
    
    (code, out, err) = subproc.execute(["fitsverify",fullpathfilename], log=log)
    if code != 0:
        hdulist = pyfits.open(fullpathfilename)
        hdulist.verify('fix')
        hdulist.writeto(path+'/'+fixedfile,output_verify='fix')
        (codefix, outfix, errfix) = subproc.execute(["fitsverify",path+'/'+fixedfile], log=None)
        if code != 0:
            os.remove(path+'/'+fixedfile)
            raise error.CDBSError(outfix,errfix,codefix)
        newerr = "Potential fix for FITS errors in "+fixedfile+".\n"+err
        helper.updatelog(log,newerr)
        raise error.CDBSError(out,newerr,code)
    helper.updatelog(log,out)
    helper.updatelog(log,"\n*** check FITS standard finsihed ***\n")
    return


# == Step 5 ==
def CDBS_certify(filename,log=None):
    """ 
    Run the CDBS certify tool on the FITS files or GEIS header files.
    """
    helper.updatelog(log,"\n*** CDBS certify started ***\n")
    (code, out, err) = subproc.execute(["certify",filename], log=log)
    if code != 0:
        helper.updatelog(log,err)
        raise error.CDBSError(out,err,code)
    helper.updatelog(log,"\n*** CDBS certify finished ***\n")
    return 

# == Step 6 ==
def create_load_file(filename,log=None):
    """
    Create the "load" files.
    """
    helper.updatelog(log,"\n*** create load file started ***\n")
    (code, out, err) = subproc.execute(["mkload",filename], log=log)
    if code != 0:
        helper.updatelog(log,err)
        raise error.CDBSError(out,err,code)
    helper.updatelog(log,"\n*** create load file finished ***\n")
    return

# == Step 7 ==
def populate_load_file(fitsfile, filename, comments=True, change_level="",
                       opus_flag='Y', pedigree="", log=None):
    """
    Populate the "load" files.
    """
    helper.updatelog(log,"\n*** populate load file started ***\n")
    try:
        setlodkeywd.setlodkeywd(fitsfile, filename, comments, change_level,
                                  opus_flag, pedigree)
    except IOError:
        errorstr = "Unable to open required files for "+os.path.basename(str(filename))+".\n"
        outstr = errorstr
        helper.updatelog(log, errorstr)
        raise error.CDBSError(outstr,errorstr,error.PYERRORCODE)

    helper.updatelog(log,"\n*** populate load file finished ***\n")

# == Step 8 ==
def certify_load_file(lodfilename,log=None):
    """
    
    Certify the "load" files.

    """
    helper.updatelog(log,"\n*** Certify load file started ***\n")
    (code, out, err) = subproc.execute(["certify",lodfilename], log=log)
    if code != 0:
        helper.updatelog(log, err)
        raise error.CDBSError(out,err,code)
    helper.updatelog(log,"\n*** Certify load file finished ***\n")
    return None


# == Step 8+ ==
def expload_load_file(lodfilename, instrument, newlodfilename=None, log=None):
    """
    
    expload_load_file: Expand image reference files to
    list all actual values in the LOD files replacing
    the wildcards from the mkload step.
    
    If the newlodfilename is not defined then the exisitng
    lodfile will be overwritten.
    
    """
    if newlodfilename == None:
        newlodfilename = lodfilename

    thisdir = helper.getcurdir()
    (fullpathfilename, path, parsedfilename) = helper.fileinfo(lodfilename)
    os.chdir(path)

    helper.updatelog(log,"\n*** EXPLOAD load file started ***\n")
    (code, out, err) = subproc.execute(["expload", 
                                        parsedfilename, newlodfilename,
                                        CDBSDATA+'/'+instrument+'.rule'], 
                                       log=log)
    if code != 0:
        helper.updatelog(log, err)
        raise error.CDBSError(out,err,code)
    
    os.chdir(thisdir)
    helper.updatelog(log,"\n*** EXPLOAD load file finished ***\n")
    return None
    
# == Step 9 ==
def run_check_load(filename,log=None):
    """
    
    run_check_load: Run application check_load.

    """
    helper.updatelog(log, "\n*** check_load started ***\n")

    thisdir = helper.getcurdir()
    (fullpathfilename, path, parsedfilename) = helper.fileinfo(filename)
    os.chdir(path)

    (code, out, err) = subproc.execute(["check_load", parsedfilename], log=log)
    if code != 0:
        helper.updatelog(log, err)
        raise error.CDBSError(out, err, code)
    
    os.chdir(thisdir)
    helper.updatelog(log, "\n*** check_load finished ***\n")

    return None

# == Step 10 ==
def rename_ref_files(filename,log=None):
    """
    
    Purpose
    =======
    Rename the pipeline reference file to have a unique name identifier.
    
    * In the case of SYNPHOT files, make sure that the throughput or bandpass
      files have a number that is greater than the last delivered version of
      the file.
    """
    helper.updatelog(log, "\n*** rename_ref_files started ***\n")

    thisdir = helper.getcurdir()
    (fullpathfilename, path, parsedfilename) = helper.fileinfo(filename)
    os.chdir(path)

    (code, out, err) = subproc.execute(["uniqname", helper.getlodfilename(parsedfilename)], log=log)
    if code != 0:
        helper.updatelog(log, err)
        raise error.CDBSError(out, err, code)
    
    newfilename = err.split()[4][:-1]
    os.chdir(thisdir)
    helper.updatelog(log, "\n*** rename_ref_files finished ***\n")
    
    return newfilename


# == Step 12 ==
def copy_to_delivery_dir(inlist,deliverydir=DELIVERYDIR,log=None):
    """
    
    Purpose
    =======
    Move the FITS and LOD files to a delivery directory
    
    """
    
    # Accept single file name or Python list of file names as input
    if isinstance(inlist,list):
        filelist = inlist
    else:
        filelist=[]
        filelist.append(inlist)
        
    helper.updatelog(log,"\n*** Copy FITS and LOD files to delivery area started ***\n")
    for filename in filelist:
        # Copy FITS File
        (code, out, err) = subproc.execute(["cp", filename, deliverydir+"/."], log=log)
        if code != 0:
            helper.updatelog(log,err)
            raise error.CDBSError(out, err, code)
        helper.updatelog(log,out)
        # Copy related LOD file
        lodname = filename.split('.')
        lodname[-1] = 'lod'
        lodfilename = ".".join(lodname)
        (code, out, err) = subproc.execute(["cp", lodfilename, deliverydir+"/."], log=log)
        if code != 0:
            helper.updatelog(log,err)
            raise error.CDBSError(out, err, code)
        helper.updatelog(log,out)
    helper.updatelog(log,"\n*** COPY FITS and LOD files to delivery area finsihed ***\n")
    return 

def dms_delivery(deliverydir,workingdir, log=None):
    """
    
    Purpose
    =======
    Deliver files to the DMS.

    """
    helper.updatelog(log,"\n*** Begin DMS_DELIVERY using SENDIT ***\n")

    thisdir = helper.getcurdir()
    
    os.chdir(deliverydir)
    (code, out, err) = subproc.execute(["sendit", ">>&", workingdir+"/delivery.log"], log=log)
    if code != 0:
        helper.updatelog(log,err)
        raise error.CDBSError(out, err, code)

    os.chdir(thisdir)
    helper.updatelog(log,"\n*** Completed DMS_DELIVERY using SENDIT ***\n")
    
def check_delivery_success(workingdir,logname,log=None):
    """
    
    Purpose
    =======
    Need to check the delivery.log file created by sendit to verify that the delivery
    script ran successfully.  Otherwise the delete script must be run to un-lock
    the database and remove the problem delivery.
    
    """
    helper.update(log,"\n*** Completed CHECK_DELIVERY_SUCCESS ***\n")

    # Determine current directory
    thisdir = helper.getcurdir()

    # Change to the WORKINGDIR directory
    os.chdir(workingdir)
    
    # Read each line of the log file
    for line in logname:
        tokens = (line.strip()).split()
        
    os.chdir(thisdir)
    helper.update(log,"\n*** Completed CHECK_DELIVERY_SUCCESS ***\n")

# == Step 13 ==
def deliver_delivery_form(filename,log=None):
    """
    
    Purpose
    =======
    Fill out the template delivery form and e-mail it to the DMS
    (if OPUS_FLAG == Y).
    
    """
    pass

